<?php

namespace NoccyLabs\Screen;

use NoccyLabs\EventEmitter\EventEmitterTrait;

class Screen
{
    use EventEmitterTrait;

    private int $lines = 0;

    private int $columns = 0;

    private bool $initialized = false;

    public function __construct()
    {
        $this->measureTerminalSize();
        pcntl_signal(SIGWINCH, [ $this, "measureTerminalSize" ]);
    }

    public function setup()
    {
        if ($this->initialized) return;
        $this->measureTerminalSize();
        passthru("tput smcup");
        echo "\e[H\e[0m\e[2K";
    }

    public function shutdown()
    {
        if ($this->initialized) return;

        passthru("tput rmcup");
    }

    public function measureTerminalSize()
    {
        $this->lines = intval(exec("tput lines"));
        $this->columns = intval(exec("tput cols"));

        $this->emitEvent("resize", [
            'lines' => $this->lines,
            'columns' => $this->columns
        ]);
    }

    public function write(string|Buffer $content)
    {
        if ($content instanceof Buffer) {
            echo $content->flush(true);
        } else {
            echo $content;
        }
    }

}
